package org.sonatype.plugins.sisu.assembler;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;

import org.sonatype.aether.RepositoryException;
import org.sonatype.aether.collection.DependencyGraphTransformationContext;
import org.sonatype.aether.collection.DependencyGraphTransformer;
import org.sonatype.aether.graph.Dependency;
import org.sonatype.aether.graph.DependencyNode;
import org.sonatype.aether.util.graph.DefaultDependencyNode;

/**
 * A dependency transformer based on dependency scopes. It eliminates from the tree dependencies that does not match
 * expected scopes.<em>Note:</em> This filter does not assume any relationships between the scopes. In particular, the
 * transformer is not aware of scopes that logically include other scopes.
 * 
 * @author Alin Dreghiciu
 * @see Dependency#getScope()
 */
public class ScopeDependencyGraphTransformer
    implements DependencyGraphTransformer
{

    private final Collection<String> included = new HashSet<String>();

    private final Collection<String> excluded = new HashSet<String>();

    /**
     * Creates a new transformer using the specified includes and excludes.
     * 
     * @param included The set of scopes to include, may be {@code null} or empty to include any scope.
     * @param excluded The set of scopes to exclude, may be {@code null} or empty to exclude no scope.
     */
    public ScopeDependencyGraphTransformer( final Collection<String> included, final Collection<String> excluded )
    {
        if ( included != null )
        {
            this.included.addAll( included );
        }
        if ( excluded != null )
        {
            this.excluded.addAll( excluded );
        }
    }

    /**
     * Creates a new transformer using the specified excludes.
     * 
     * @param excluded The set of scopes to exclude, may be {@code null} or empty to exclude no scope.
     */
    public ScopeDependencyGraphTransformer( final String... excluded )
    {
        if ( excluded != null )
        {
            this.excluded.addAll( Arrays.asList( excluded ) );
        }
    }

    public DependencyNode transformGraph( final DependencyNode node, final DependencyGraphTransformationContext context )
        throws RepositoryException
    {
        DefaultDependencyNode parent = null;
        if ( !accept( node ) )
        {
            parent = new DefaultDependencyNode();
        }

        transformGraph( node, parent );

        return parent != null ? parent : node;
    }

    private void transformGraph( final DependencyNode node, final DependencyNode parent )
    {
        final boolean accepted = accept( node );
        for ( final DependencyNode child : new ArrayList<DependencyNode>( node.getChildren() ) )
        {
            transformGraph( child, node );
        }
        if ( !accepted && parent != null )
        {
            parent.getChildren().remove( node );
            for ( final DependencyNode child : node.getChildren() )
            {
                parent.getChildren().add( child );
            }
        }
    }

    public boolean accept( final DependencyNode node )
    {
        final Dependency dependency = node.getDependency();

        if ( dependency == null )
        {
            return true;
        }

        final String scope = node.getDependency().getScope();

        final boolean accepted = ( included.isEmpty() || included.contains( scope ) )
            && ( excluded.isEmpty() || !excluded.contains( scope ) );

        return accepted;
    }

}
